/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.localizer.basic;

import mir.session.Request;
import mir.session.Response;
import mir.session.Session;
import mir.session.SessionExc;
import mir.session.SessionFailure;
import mircoders.entity.EntityContent;
import mircoders.global.MirGlobal;
import mircoders.module.ModuleArticleType;
import mircoders.storage.DatabaseContent;

public class MirBasicTranslationPostingHandler extends MirBasicChildArticlePostingHandler {
  public MirBasicTranslationPostingHandler() {
    super();

    setResponseGenerators(
        configuration.getString("Localizer.OpenSession.translation.EditTemplate"),
        configuration.getString("Localizer.OpenSession.translation.DupeTemplate"),
        configuration.getString("Localizer.OpenSession.translation.UnsupportedMediaTemplate"),
        configuration.getString("Localizer.OpenSession.translation.DoneTemplate"));
  }

  protected void initialRequest(Request aRequest, Session aSession, Response aResponse) throws SessionExc, SessionFailure {
    initializeSession(aRequest, aSession);
    initializeResponseData(aRequest, aSession, aResponse);

    if (aRequest.getParameter("title")!=null)
      subsequentRequest(aRequest, aSession, aResponse);
    else
      makeInitialResponse(aRequest, aSession, aResponse);
  }

  public void finalizeArticle(Request aRequest, Session aSession, EntityContent anArticle) throws SessionExc, SessionFailure {
    String is_html=anArticle.getFieldValue("is_html");
    super.finalizeArticle(aRequest, aSession, anArticle);
    anArticle.setFieldValue("is_html", is_html);
    try {
      anArticle.setFieldValue("to_article_type", new ModuleArticleType().articleTypeIdForName("translation"));
    }
    catch (Throwable t) {
      throw new SessionFailure(t);
    }
  }

  protected void initializeSession(Request aRequest, Session aSession) throws SessionExc, SessionFailure {
    super.initializeSession(aRequest, aSession);

    try {
      String parentId = aRequest.getParameter("to_content");
      aSession.setAttribute("parent",
                            MirGlobal.localizer().dataModel().adapterModel().makeEntityAdapter("content", DatabaseContent.getInstance().selectById(parentId))
                            );
    }
    catch (Throwable t) {
      throw new SessionFailure(t);
    }
  }

  protected void initializeResponseData(Request aRequest, Session aSession, Response aResponse) throws SessionExc, SessionFailure {
    try {
      super.initializeResponseData(aRequest, aSession, aResponse);
      aResponse.setResponseValue("parent", aSession.getAttribute("parent"));
    }
    catch (Throwable t) {
      throw new SessionFailure(t);
    }
  }

}
